! WHIZARD 1.22 Apr 15 2002
! 
! (C) 1999-2001 by Wolfgang Kilian <kilian@particle.uni-karlsruhe.de>
!
! WHIZARD is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! WHIZARD is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This file has been stripped of most comments.  For documentation, refer
! to the source 'whizard.nw'

module jetset_interface

  use kinds, only: default !NODEP!
  use kinds, only: double !NODEP!
  use file_utils, only: free_unit !NODEP!
  use diagnostics, only: msg_buffer, msg_message, msg_fatal
  use limits, only: PYTHIA_EXTERNAL_PROCESS, MB_PER_FB_EXPONENT, BUFFER_SIZE
  use parser, only: get_integer, lex_clear

  implicit none
  private

  integer, parameter, public :: PYGIVE_LEN = 100

  public :: jetset_init, pythia_init
  public :: jetset_fragment
  public :: pythia_fragment
  public :: pythia_up_fill
  public :: jetset_open_write, jetset_close
  public :: jetset_write
  public :: jetset_suppress_banner
  public :: pythia_print_statistics

  logical, public :: new_event_needed_pythia = .true.
  integer, public :: number_events_whizard = 0
  integer, public :: number_events_pythia = 0
  integer, dimension(2), public :: number_events_written = 0
!  integer :: mstp61, mstp71
  integer :: pythia_isub
  real(kind=double) :: pythia_sqrts, pythia_integral
  integer :: pythia_nup
  integer, dimension(20,7) :: pythia_kup
  real(kind=double), dimension(20,5) :: pythia_pup
!  integer :: pythia_nfup
!  integer, dimension(10,2) :: pythia_ifup
!  real(kind=double) :: pythia_q2up
  save

contains

  subroutine jetset_init (chin)
    character(len=PYGIVE_LEN), intent(in) :: chin
    if (chin /= ' ') then
       call msg_message (" PYTHIA parameters modified:")
       call pygive (chin)
    end if
  end subroutine jetset_init

  subroutine pythia_init (n_external, process_id, beam, target, &
       &                  fixed_energy, sqrts, integral, &
       &                  pyproc, chin, show_pymaxi)
    integer, intent(in) :: n_external
    character(len=*), intent(in) :: process_id, beam, target
    logical, intent(in) :: fixed_energy
    real(kind=default), intent(in) :: sqrts, integral
    character(len=BUFFER_SIZE), intent(in) :: pyproc
    character(len=PYGIVE_LEN), intent(in) :: chin
    logical, intent(in) :: show_pymaxi
    character(len=BUFFER_SIZE) :: buffer
    character(len=len(beam)) :: beam_test1, beam_test2
    integer :: isub_test
    common /PYPARS/ mstp(200),parp(200),msti(200),pari(200)
    integer :: mstp, msti
    real(kind=double) :: parp, pari
    common /PYSUBS/ msel,mselpd,msub(500),kfin(2,-40:40),ckin(200)
    integer :: msel, mselpd, msub, kfin
    real(kind=double) :: ckin
    call msg_message 
    call msg_message (" Initializing PYTHIA ...")
    msel=0            ! No extra processes
    isub_test = 24    ! ZH subprocess
    msub(isub_test)=1
    mstp(122)=0       ! First pyinit call silent
    beam_test1="e+"
    beam_test2="e-"
    call pyinit ("CMS", beam_test1, beam_test2, 5000._double)
    call pyevnt
    msub(isub_test)=0
    mstp(122)=1       ! Now print it again
    if (n_external > 20) then
       write (msg_buffer, "(A,1x,I5)") &
            & " Number of external particles:", n_external
       call msg_message
       call msg_fatal &
            & (" PYTHIA fragmentation is possible for max. 20 particles.")
    end if
    pythia_nup = n_external
    pythia_isub = PYTHIA_EXTERNAL_PROCESS
    pythia_sqrts = sqrts
    pythia_integral = integral / 10.d0**MB_PER_FB_EXPONENT
    call msg_message (" PYTHIA parameters modified:")
    call set_processes (pyproc)
    call pyupin (pythia_isub, trim(process_id), pythia_integral)
    if (chin /= ' ') call pygive (chin)           ! User intervention
!    mstp61 = mstp(61)
!    mstp71 = mstp(71)
    if (.not.fixed_energy) then
       pythia_sqrts = pythia_sqrts &
            &         * 1.00000000001_double      ! suppress x=1 warnings
    end if
    call pygive ("MSTP(11)=0; MSTP(61)=0; MSTP(71)=0") ! Suppress ISR/FSR
    buffer = " "
    write (buffer, "(I3)")  pythia_isub
    call pygive ("MSEL=0; MSUB("//trim(buffer)//")=1") ! select user process
    if (.not.show_pymaxi)  call pygive ("MSTP(122)=0")
    call pyinit ("CMS", beam, target, pythia_sqrts)
    new_event_needed_pythia = .true.
    number_events_whizard = 0
    number_events_pythia = 0
  contains
    subroutine set_processes (pyproc)
      character(len=*), intent(in) :: pyproc
      integer :: u, iostat, process
      character(len=BUFFER_SIZE) :: buffer
      u = free_unit ()
      open (u, status="scratch")
      write (u, "(A)") pyproc
      rewind (u)
      SCAN_TOKENS: do
         process = get_integer (u, iostat=iostat)
         if (iostat /= 0)  exit SCAN_TOKENS
         buffer = " "
         write (buffer, "(I4)")  process
         call pygive ("MSUB("//trim(buffer)//") = 1")
      end do SCAN_TOKENS
      close (u, status="delete")
      call lex_clear
    end subroutine set_processes

  end subroutine pythia_init

  subroutine jetset_fragment (color_flow, anticolor_flow)
    integer, dimension(:), intent(in) :: color_flow, anticolor_flow
    integer, dimension(size(color_flow)) :: cflow, aflow
    integer :: i
    call pyhepc (2)
    cflow = color_flow
    aflow = anticolor_flow
    do i=1, size(cflow)
       if (cflow(i)/=0 .and. aflow(i)==0) then
          call make_color_string (i, cflow, aflow)
       end if
    end do
    do i=1, size(cflow)
       if (aflow(i)/=0 .and. cflow(i)==0) then
          call make_color_string (i, aflow, cflow)
       end if
    end do
    do i=1, size(cflow)
       if (cflow(i)/=0 .and. aflow(i)/=0) then
          call make_color_string (i, cflow, aflow)
       end if
    end do
    call pyexec ()
    call pyhepc (1)
  contains
  subroutine make_color_string (i0, flow1, flow2)
    integer, intent(in) :: i0
    integer, dimension(:), intent(inout) :: flow1, flow2
    integer, dimension(size(flow1)) :: ijoin
    integer :: njoin
    integer :: i1, i2, j
    i1 = i0
    ijoin = 0
    njoin = 0
    do j=1, size(ijoin)
       if (i1 <= 2) exit
       ijoin(j) = i1 - 2
       njoin = njoin + 1
       i2 = i1
       i1 = flow1(i2)
       flow1(i2) = 0
       flow2(i2) = 0
    end do
    if (njoin > 1)  call pyjoin (njoin, ijoin)
  end subroutine make_color_string
  end subroutine jetset_fragment

  subroutine pythia_fragment (m, p, code, color_flow, anticolor_flow)
    real(kind=default), dimension(:), intent(in) :: m
    real(kind=default), dimension(:,0:), intent(in) :: p
    integer, dimension(:), intent(in) :: code, color_flow, anticolor_flow
    integer :: n
    common /PYPARS/ mstp(200),parp(200),msti(200),pari(200)
    integer :: mstp, msti
    real(kind=double) :: parp, pari
!    mstp(61) = mstp61
!    mstp(71) = mstp71
    if (new_event_needed_pythia) then
       n = pythia_nup
       pythia_kup (:n,1) = 1
       pythia_kup (:n,2) = code (:n)
       pythia_kup (:n,3) = 0
       pythia_kup (:2,4)  = 0
       pythia_kup (3:n,4) = color_flow (3:n)
       pythia_kup (1:2,5) = 0
       pythia_kup (3:n,5) = anticolor_flow (3:n)
       pythia_kup (1:2,6) = anticolor_flow (1:2)
       pythia_kup (3:n,6) = 0
       pythia_kup (1:2,7) = color_flow (1:2)
       pythia_kup (3:n,7) = 0
       pythia_pup (1:n,1:3) = p(1:n,1:3)
       pythia_pup (1:n,4)   = p(1:n,0)
       pythia_pup (1:n,5)   = m(1:n)
       new_event_needed_pythia = .false.
    end if
    call pyevnt ()
    call pyhepc (1)
    if (new_event_needed_pythia) then
       number_events_whizard = number_events_whizard + 1
    else
       number_events_pythia = number_events_pythia + 1
    end if
  end subroutine pythia_fragment

  subroutine pythia_up_fill (integral)
    real(kind=double), intent(out) :: integral
    integer :: nup, kup, nfup, ifup
    real(kind=double) :: pup, q2up
    common /PYUPPR/ nup, kup(20,7), nfup, ifup(10,2), pup(20,5), q2up(0:10)
    save /PYUPPR/
    common /PYPARS/ mstp(200),parp(200),msti(200),pari(200)
    integer :: mstp, msti
    real(kind=double) :: parp, pari
!    mstp(61) = 0
!    mstp(71) = 0
    nup = pythia_nup
    kup = pythia_kup
    pup = pythia_pup
    integral = pythia_integral
!    nfup = pythia_nfup
!    ifup = pythia_ifup
!    q2up = pythia_q2up
  end subroutine pythia_up_fill

  subroutine jetset_open_write (unit, file)
    integer, intent(in) :: unit
    character (len=*), intent(in) :: file
    external F77OPN
    call F77OPN (unit, trim(file))
    number_events_written = 0
  end subroutine jetset_open_write

  subroutine jetset_close (unit)
    integer, intent(in) :: unit
    external F77CLS
    call F77CLS (unit)
  end subroutine jetset_close

  subroutine jetset_write (unit, mlist)
    integer, intent(in) :: unit, mlist
    common /PYDAT1/ mstu(200), paru(200), mstj(200), parj(200)
    integer :: mstu, mstj
    real(kind=double) :: paru, parj
    integer :: mstu11, nev
    character(len=BUFFER_SIZE) :: buffer
    external F77WCH
    call pyhepc (2)
    mstu11 = mstu(11)
    mstu(11) = unit
    buffer = " "
    if (unit==6) then
       number_events_written(1) = number_events_written(1) + 1
       nev = number_events_written(1)
    else
       number_events_written(2) = number_events_written(2) + 1
       nev = number_events_written(2)
    end if
    if (new_event_needed_pythia) then
       write (buffer, "(1x,A,1x,I12,3x,A)") "*** Event #", nev, "(WHIZARD):"
    else
       write (buffer, "(1x,A,1x,I12,3x,A)") "*** Event #", nev, "(PYTHIA):"
    end if
    call F77WCH (unit, " ")
    call F77WCH (unit, " ")
    call F77WCH (unit, " ")
    call F77WCH (unit, trim(buffer))
    call jetset_suppress_banner
    call pylist (mlist)
    mstu(11) = mstu11
  end subroutine jetset_write

  subroutine jetset_suppress_banner
    common /PYDAT1/ mstu(200), paru(200), mstj(200), parj(200)
    integer :: mstu, mstj
    real(kind=double) :: paru, parj
    mstu(12) = 0
  end subroutine jetset_suppress_banner
  subroutine pythia_print_statistics
    call pystat (1)
  end subroutine pythia_print_statistics


end module jetset_interface

subroutine pyupev (isub_dummy, sigev)
  use kinds, only: double !NODEP!
  use jetset_interface, only: pythia_up_fill, new_event_needed_pythia !NODEP!
  implicit none
  integer, intent(in) :: isub_dummy
  real(kind=double), intent(out) :: sigev
  real(kind=double) :: integral
  call pythia_up_fill (integral)
  sigev = integral
  new_event_needed_pythia = .true.
end subroutine pyupev

